#!/usr/bin/env python3
"""
Setup fallback pour compatibilité legacy
GestVenv utilise pyproject.toml comme configuration principale
"""

from setuptools import setup

# Configuration via pyproject.toml
setup()